
# \Weaver is an extension to IBM DevOps efforts allowing to formalize the bill of material that is important in DevOps, 
# i.e., the application, the infrastructure and the environment and creating/maintaining/testing and debugging 
# infrastructures as code. Our key research challenges and goals are the development of a semantically rich, 
# extensible, model-driven language for infrastructure development with advanced capabilities such as validation, 
# testing/debugging and integration with runtime analytics (e.g., performance analysis & monitoring).
#
# == Roadmap
# 
# {Weaver::Core} provides a model for storing metadata about deployments.
#
# {Weaver::Dsl} provides a parser for .weaver files that creates model objects.
#
# {Weaver::Chef} provides utilities to create \Weaver models from Chef cookbooks and to validate Weaver models for drift against cookbooks.
#
# {Weaver::Cli} provides a command line interface for \Weaver operations such as creating .weaver files and deploying .weaver files
#
# {Weaver::Generators} provide functions to create Weaver models
#
# {Weaver::Iwd} provides functions to import from and deploy 
# to {http://www-01.ibm.com/software/webservers/workload-deployer/ IBM Workload Deployer}
#
# {Weaver::Transforms} provides functions to duplicate or transform a \Weaver model
#
# {Weaver::Utils} provides utility functions for operating on \Weaver models
#
# {Weaver::Zephyr} provides utility functions to import from 
# {http://www-01.ibm.com/software/rational/products/swarchitect/deployment/ Rational Software Architect Extension for Deployment Planning} 
module Weaver
  
  # Weaver's \Core module defines model elements that Weaver knows about and can validate and deploy from.
  # 
  # A Topology represents a set of machines called Nodes.  Both virtual machine instances and physical machines are Nodes.
  # A Node can contain Component s which represent software installed or to-be-installed on a Node.  Automation is used to
  # install Components and bring Nodes into compliance with intended Attribute values. 
  module Core
    
    TYPES_DIR = 'types'
    VALIDATORS_DIR = 'validators'
    RULES_DIR = 'rules'
    TOPOLOGIES_DIR = 'topologies'
    AUTOMATION_DIR = 'automation'
                
    # File suffix for each Weaver files.
    WEAVER_SUFFIX = '.weaver'
    
  end # module Core
end # module Weaver
