require 'weaver/core/base.rb'

module Weaver
  module Core
    # A mixin for classes such as Application which may contain User
    # 
    # Author:: Ed Snible
    # Copyright:: IBM
    module UserContainerMixin
      # Let all classes that include UserContainerMixin have a method to return their users
      def self.included(baseclazz)
        baseclazz.class_exec {
          # Hash of id -> User
          def users
            # Since mixins have no initialize() to set up @users, initialize @users lazily
            @users ||= {} unless frozen?
            return @users || {}
          end
        }
        
        baseclazz.register_initialize_copy_handler(:initialize_copy_users)
        baseclazz.register_validate_handler(:validate_users)
        baseclazz.register_child_enumerator(:users)           
        
      end
      
      # Deeply copy @users
      def initialize_copy_users(other_user_container)
        # At this point, self.initialize_copy should have cloned @users but the values are shallow.
        @users = {}
        other_user_container.users.each do |id, user|
          dup = user.clone()
          dup.scope = self
          @users[id] = dup
        end
      end

      # Validate @users      
      # * +out_messages+ ProblemStatus will be pushed onto this for any validation message
      # * +desired_phases+ optional Array which may be +[:predefined]:
      def validate_users(out_messages, desired_phases = nil)
        users.each { |id, user| user.validate_collect(out_messages, desired_phases) }
      end
    end
    
  end
end