# storage_modifier.rb

require 'weaver/transforms/duplicate'

module Weaver
  module Deployers
    module Execution
        
      # Helper methods to create Chef automation to mount associated networking links
      class StorageModifier
        include Weaver::Core
        
        # +node+ Node
        def self.add_weaver_storage_role(node, deploy_parameters)
          # Because links might be in the application, and not copied to the infrastructure, look through
          # the entire model for links
          storage_links = node.root_entity.flatten_children.
             select { |child| child.kind_of?(Link) && child.tags.include?(:storage) && child.source.equal?(node) && child.target != nil }
          storage_links.each do |storage_link|
            # Add automation
            automation = Weaver::Transforms::Duplicate::duplicate(weaver_storage)
            automation.id = storage_link.id # TODO check for collision
            automation.scope = node
            # Add the new automation, but instead of just adding the key redo the whole hash so the storage role is first
            orig_automation = node.automation.clone
            node.automation.clear
            node.automation[automation.id] = automation
            node.automation.merge!(orig_automation)
            
            automation.deps_file_url = deploy_parameters[:deps_file_url]
              
            # Customize automation
            # TODO simplify the unless once we have a first-class storage link
            automation.weaver_storage_device = storage_link.dev_name unless storage_link.amd[:dev_name].nil?
            automation.weaver_storage_mount_point = storage_link.mount_point unless storage_link.amd[:mount_point].nil?
            automation.weaver_storage_fstype = storage_link.target.fstype unless storage_link.target.amd[:fstype].nil? || storage_link.target.fstype.nil? 
            automation.weaver_storage_format = storage_link.target.format unless storage_link.target.amd[:format].nil? || storage_link.target.format.nil? 
            automation.weaver_storage_user = storage_link.storage_user unless storage_link.target.amd[:storage_user].nil? || storage_link.target.storage_user.nil? 
            automation.weaver_storage_group = storage_link.storage_group unless storage_link.target.amd[:storage_group].nil? || storage_link.target.storage_group.nil? 
            automation.weaver_storage_attach_timeout = storage_link.storage_attach_timeout unless storage_link.target.amd[:storage_attach_timeout].nil? || storage_link.target.storage_attach_timeout.nil? 
          end
        end
        
        protected
        
        def self.weaver_storage
          @storage_model = create_weaver_storage if @storage_model.nil?
          return @storage_model
        end
        
        def self.create_weaver_storage
          root = Weaver::Dsl::WeaverParser.new.parse(File.expand_path('../weaver_storage_template.weaver', __FILE__))
          return root.weaver_storage
        end
        
      end # class StorageModifier
    end # module Execution
  end # module Deployers
end # module Weaver