require 'weaver/core/firewall'
require 'weaver/dsl/abstract_statement'

module Weaver
  module Dsl
    # FirewallStatement implements the processing of the language element 'firewall'.
    #
    # Author:: Ed Snible
    # @see Firewall
    class FirewallStatement < AbstractStatement

      # * +parent+ The enclosing AbstractStatement
      def initialize(parent)
        @model = Firewall.new()
        @parent = parent
      end

      # Support either a list of open TCP ports, or a Hash of :port and :netmask
      # The list can include Fixnum and Range only
      def allow_tcp(firewalldata)
        if firewalldata.kind_of?(Hash)
          ports = firewalldata.delete(:port)
          netmask = firewalldata.delete(:netmask)
          direction = firewalldata.delete(:direction)
          return @parent.model.add_validation_failure("Unexpected firewall attribute #{firewalldata.keys.to_a[0]}") if firewalldata.size > 0
        else
          ports = firewalldata
          netmask = nil
          direction = nil
        end
        
        @model.tcp.push FirewallRule.new(ports, netmask, direction)
      end
      
      # Support either a list of open UDP ports, or a Hash of :port and :netmask
      # The list can include Fixnum and Range only
      def allow_udp(firewalldata)
        if firewalldata.kind_of?(Hash)
          ports = firewalldata.delete(:port)
          netmask = firewalldata.delete(:netmask)
          direction = firewalldata.delete(:direction)
          return @parent.model.add_validation_failure("Unexpected firewall attribute #{firewalldata.keys}") if firewalldata.size > 0
        else
          ports = firewalldata
          netmask = nil
          direction = nil
        end
        
        @model.udp.push FirewallRule.new(ports, netmask, direction)
      end
      
    end
  end

end