
module Weaver
  module Utils
    # Property indent Weaver code.  This code typically only formats code generated by Weaver's templates,
    # and this implementation can't properly indent a full Weaver document including late_bindings or other
    # Ruby control flow.
    module Indent
      
      # Put PER_LEVEL * level before each line
      # @private
      PER_LEVEL = '  '
      
      def self.fixup(s)
        indented_code = []
        level = 0
        s.each_line do |line|
          line.strip!
          # If the line is non-empty
          level = [level - outdent_effect(line), 0].max
          if line.match(/\S/)
            indented_code << PER_LEVEL * level + line
          else
            indented_code << ''
          end
          level = [level + indent_effect(line), 0].max
        end
        
        return indented_code.join("\n") + "\n"
      end
      
      # How much effect does this line have on tabs?
      # @private
      # Note that we ignore braces in quotes, which are rare in Weaver documents.
      # A better version would throw away trailing comments, String constants, and RegExp constants
      # before counting.  Weaver doesn't generate do/end or any other Ruby control flow unless
      # there are embedded procs which aren't usually generated anyway.
      def self.indent_effect(line)
        return line.count('{')
      end
      
      # How much effect does this line have on tabs?
      # @private
      # Note that we ignore braces in quotes, which are rare in Weaver documents.
      # A better version would throw away trailing comments, String constants, and RegExp constants
      # before counting.  Weaver doesn't generate do/end or any other Ruby control flow unless
      # there are embedded procs which aren't usually generated anyway.
      def self.outdent_effect(line)
        return line.count('}')
      end
    end
  end

end